#include <cstdlib>
#include <algorithm>
#include <iostream>
#include "VectorWindow.h"

enum menuenties { ACTION,TRANSLATE,ZOOM_IN,ZOOM_OUT,QUIT };

///
class DrawPoint
{
  public:

  ///
  void operator()(const Point &p)
  {
    glVertex2d(p.x,p.y);
  }
}///
;

///
VectorWindow::VectorWindow(GlutAdmin &admin,int initW,int initH,int initX, int initY, char *title)
{
   width = initW;
   height = initH;
//   rx0 = ry0 = 0;
   op_mode = 0;
   scaling = false;

   glutInitDisplayMode(GLUT_RGBA | GLUT_DOUBLE );
   glutInitWindowSize(width,height);
   glutInitWindowPosition(initX,initY);

   admin.CreateWindow(title,this);

   int submenu = glutCreateMenu(admin.MenuFunc);
   glutAddMenuEntry("Translate",TRANSLATE);
   glutAddMenuEntry("Zoom in",ZOOM_IN);
   glutAddMenuEntry("Zoom out",ZOOM_OUT);
   int mainmenu = glutCreateMenu(admin.MenuFunc);
   glutAddSubMenu("Action",submenu);
   glutAddMenuEntry("Quit",QUIT);

   glutAttachMenu(GLUT_RIGHT_BUTTON);
}

///
VectorWindow::~VectorWindow()
{
   glutDestroyWindow(ID);
}

///
void VectorWindow::MenuFunc(int value)
{ 
  if (value == QUIT) exit(EXIT_SUCCESS);
  op_mode = value;
}

void VectorWindow::MouseFunc(int button, int state, int x, int y)
{
  if (button != GLUT_LEFT_BUTTON) return;
  if (state == GLUT_DOWN)
  {
    mx0 = x; my0 = height-y;
  }
  if (op_mode == ZOOM_IN || op_mode == ZOOM_OUT)
	scaling = (state == GLUT_DOWN);
}

///
void VectorWindow::MotionFunc(int x, int y)
{
  if (op_mode == TRANSLATE)
  {
	llcorner.x += (mx0-x)/scale;
	llcorner.y += (my0-height+y)/scale;

	mx0 = x; 
    my0 = height-y;

    glutPostRedisplay();
  }
}

///
void VectorWindow::IdleFunc()
{
  if (scaling)
  {
    if (op_mode == ZOOM_IN)
	{
	  scale *= 1.05;
	  llcorner.x += mx0*0.05/scale;
	  llcorner.y += my0*0.05/scale;
	}
	if (op_mode == ZOOM_OUT)
	{
	  llcorner.x -= mx0*0.05/scale;
	  llcorner.y -= my0*0.05/scale;
	  scale /= 1.05;
	}

    glutPostRedisplay();
  }
}

///
void VectorWindow::DisplayFunc()

{ 
  glClear(GL_COLOR_BUFFER_BIT);
  glPushMatrix();
  glScaled(scale,scale,1);
  glTranslated(-llcorner.x,-llcorner.y,0.0);

  map<string,Fluss>::iterator i;
  list<Flussteil>::iterator j;

  for(i = M.begin(); i != M.end(); ++i)
  {
    Fluss &f = i->second;

	if (!f.visible) continue;
	if (f.mark)
	  glColor3f(0,1,0);
	else 
	  glColor3f(0,0,1);

	for(j = f.L.begin(); j != f.L.end(); ++j)
	{ 
	  glBegin(GL_LINE_STRIP);
      for_each(j->V.begin(), j->V.end(), DrawPoint() );
      glEnd();
	}
  }
/*
  glColor3f(1.0,0.0,0.0);
  glPointSize(3.0);
  glBegin( GL_POINTS);
  for_each(C.begin(),C.end(),DrawPoint());
  glEnd();
*/
  glPopMatrix();
  glutSwapBuffers();
}

///
void VectorWindow::ReshapeFunc(int newwidth, int newheight)
{  
  width = newwidth;
  height = newheight;

  glViewport(0,0,GLint(width),GLint(height));
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluOrtho2D(0,GLdouble(width),0,GLdouble(height));
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
}

